<?php
/* --------------------------------------------------------------
   OvistoLogoStorageReader.inc.php 2018-01-30
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/


/**
 * Class OvistoLogoStorageReader
 */
class OvistoLogoStorageReader implements OvistoLogoStorageReaderInterface
{
    /**
     * @var OvistoLogoFactory $logoFactory
     */
    protected $logoFactory;
    
    /**
     * @var \ExistingDirectory $rootDirector
     */
    protected $rootDirectory;
    
    
    /**
     * OvistoLogoStorageReader constructor.
     *
     * @param \OvistoLogoFactory $logoFactory
     * @param \ExistingDirectory $rootDirectory
     */
    public function __construct(OvistoLogoFactory $logoFactory, ExistingDirectory $rootDirectory)
    {
        $this->logoFactory = $logoFactory;
        $this->rootDirectory = $rootDirectory;
    }
    
    
    /**
     * Gets the requested logo from the filesystem.
     *
     * @param StringType $logoUrl The requested logo
     *
     * @return \OvistoLogo
     */
    public function getLogo(StringType $logoUrl)
    {
        if ($this->logoExists($logoUrl)) {
            return $this->logoFactory->createLogo($logoUrl, new StringType($this->generateFilename($logoUrl)));
        }
        
        return $this->logoFactory->createEmptyLogo();
    }
    
    
    /**
     * Checks if the requested logo is present.
     *
     * @param StringType $logoUrl The requested logo
     *
     * @return bool
     */
    protected function logoExists(StringType $logoUrl)
    {
        $filename = $this->generateFilename($logoUrl);
        
        return file_exists($this->rootDirectory->getAbsolutePath() . '/' . OvistoConfiguration::LOGO_STORAGE_DIRECTORY . $filename);
    }
    
    
    /**
     * Replaces all characters of a string that are illegal in common filesystems with an underscore (_) and returns it.
     * 
     * @param StringType $subject
     *
     * @return string
     */
    protected function generateFilename(StringType $subject)
    {
        return preg_replace('/[^a-z0-9_\.]/i', '_', $subject->asString());
    }
}